@@ -50,7 +50,6 @@ Ftp *ftp_create(void)
 	ftp->last_mkpath = 0;
 	ftp->cache = list_new((listfunc)rdir_destroy);
 	ftp->dirs_to_flush = list_new((listfunc)xfree);
-	ftp->pasvmode = false;
 	ftp->reply_timeout = 30;
 	ftp->open_timeout = 30;
 	ftp->taglist = list_new((listfunc)rfile_destroy);
@@ -448,7 +447,6 @@ static int ftp_gets(void)
 			ftp->fullcode = atoi(ftp->reply);
 	}
 #endif
-/*	c = strlen(ftp->reply);*/
 	strip_trailing_chars(ftp->reply, "\n\r");
 	ftp->code = ftp->fullcode / 100;
 	return ftp->fullcode;
@@ -721,6 +719,41 @@ static int get_password(url_t *url, const char *anonpass, bool isproxy)
 	return 0;
 }
 
+static const char *secext_name(const char *mech)
+{
+	static struct {
+		const char *short_name;
+		const char *real_name;
+	} names[] = {
+		{"krb4", "KERBEROS_V4"},
+		{"krb5", "GSSAPI"},
+/*		{"ssl", "SSL"},*/
+		{"none", "none"},
+		{0, 0}
+	};
+	int i;
+
+	for(i = 0; names[i].short_name; i++) {
+		if(strcasecmp(mech, names[i].short_name) == 0)
+			return names[i].real_name;
+	}
+
+	return 0;
+}
+
+static bool mech_unsupported(const char *mech)
+{
+#ifndef KRB4
+	if(strcasecmp(mech, "KERBEROS_V4") == 0)
+		return true;
+#endif
+#ifndef KRB5
+	if(strcasecmp(mech, "GSSAPI") == 0)
+		return true;
+#endif
+	return false;
+}
+
 int ftp_login(const char *guessed_username, const char *anonpass)
 {
 	int ptype, r;
@@ -753,23 +786,55 @@ int ftp_login(const char *guessed_username, const char *anonpass)
 	ftp->sec_complete = false;
 	ftp->data_prot = prot_clear;
 
-	if(!ftp->url->nokrb && !url_isanon(ftp->url)) {
+	/* don't use secure stuff if anonymous
+	 */
+	if(!url_isanon(ftp->url)) {
+		list *mechlist;
+		/* request a protection level
+		 */
 		if(ftp->url->protlevel) {
 			if(sec_request_prot(ftp->url->protlevel) != 0)
 				ftp_err(_("Invalid protection level '%s'\n"),
 						ftp->url->protlevel);
 		}
 
-		if(sec_login(ftp->host->hostname)) {
-			ftp_err(_("*** Using plaintext username and password ***\n"));
-			if(ftp->code == ctError
-			   && ftp->fullcode != 504 && ftp->fullcode != 534)
-				ftp->url->nokrb = true;
-		} else
+		/* get list of mechanisms to try
+		 */
+		mechlist = ftp->url->mech ? ftp->url->mech : gvDefaultMechanism;
+		if(mechlist) {
+			listitem *li = mechlist->first;
+			int ret = 0;
+			for(; li; li=li->next) {
+				const char *mech_name;
+
+				mech_name = secext_name((char *)li->data);
+				if(mech_name == 0) {
+					ftp_err(_("unknown mechanism '%s'\n"), (char *)li->data);
+					continue;
+				}
+				if(mech_unsupported(mech_name)) {
+					ftp_err(_("Yafc was not compiled with support for %s\n"),
+							mech_name);
+					continue;
+				}
+				ret = sec_login(ftp->host->hostname, mech_name);
+				if(ret == -1) {
+					if(ftp->code == ctError
+					   && ftp->fullcode != 504 && ftp->fullcode != 534)
+						url_setmech(ftp->url, "none");
+				}
+				if(ret != 1)
+					break;
+			}
+		}
+		if(ftp->sec_complete)
 			ftp_err(_("Authentication successful.\n"));
+		else
+			ftp_err(_("*** Using plaintext username"
+					  " and password ***\n"));
 	}
 #endif
-
+	
 	if(url_isanon(ftp->url))
 		fprintf(stderr, _("logging in anonymously...\n"));
 	ftp_set_tmp_verbosity(ftp->url->password ? vbError : vbCommand);
@@ -919,6 +984,8 @@ char *ftp_getcurdir(void)
 		ret = (char *)xmalloc(end-beg+1);
 		strncpy(ret, beg, end-beg);
 		stripslash(ret);
+		/* path shouldn't include any quoted chars */
+		path_dos2unix(ret);
 		return ret;
 	}
 	return xstrdup("CWD?");
@@ -929,6 +996,7 @@ static void ftp_update_curdir_x(const char *p)
 	xfree(ftp->prevdir);
 	ftp->prevdir = ftp->curdir;
 	ftp->curdir = xstrdup(p);
+	path_dos2unix(ftp->curdir);
 }
 
 static void ftp_update_curdir(void)
@@ -1342,10 +1410,10 @@ void ftp_flush_reply(void)
 		else
 			break;
 	}
-
+#if 0
 	if(ftp_loggedin())
 		ftp_chdir(ftp->curdir);
-
+#endif
 	ftp_set_close_handler();
 }
 
