@@ -26,7 +26,7 @@
 #include "commands.h"
 #include "strq.h"
 #include "shortpath.h"
-
+#include "utils.h"
 
 #ifdef HAVE_REGEX_H
 # include <regex.h>
@@ -49,6 +49,7 @@
 #define FXP_BACKGROUND (1 << 14)
 #define FXP_ASCII (1 << 15)
 #define FXP_BINARY (1 << 16)
+#define FXP_OUTPUT_FILE (1 << 17)
 
 static Ftp *fxp_target = 0;
 static bool fxp_batch = false;
@@ -65,15 +66,6 @@ static regex_t fxp_dir_rx_mask;
 static bool fxp_dir_rx_mask_set = false;
 #endif
 
-/* in put.c, FIXME: clean up! */
-char* get_mode_string(mode_t m);
-
-/* in commands.c */
-listitem *ftplist_search(const char *str);
-
-/* in cmd.c */
-void reset_xterm_title(void);
-
 static void print_fxp_syntax(void)
 {
 	printf(_("Transfers files between two remote servers.  Usage:\n"
@@ -88,17 +80,18 @@ static void print_fxp_syntax(void)
 			 "  -H, --nohup          transfer files in background (nohup mode), quits yafc\n"
 			 "  -i, --interactive    prompt before each transfer\n"
 			 "  -L, --logfile=FILE   use FILE as logfile instead of ~/.yafc/nohup/nohup.<pid>\n"
-			 "  -m, --mask=GLOB      get only files matching GLOB pattern\n"
-			 "  -M, --rx-mask=REGEXP get only files matching REGEXP pattern\n"
-			 "  -n, --newer          get file if destination is newer than source file\n"
-			 "  -o, --output=DEST    store in destination directory DEST\n"
+			 "  -m, --mask=GLOB      only transfer files matching GLOB pattern\n"
+			 "  -M, --rx-mask=REGEXP only transfer files matching REGEXP pattern\n"
+			 "  -n, --newer          transfer file if destination is newer than source file\n"
+			 "  -o, --output=DEST    store in destination file/directory DEST\n"
 			 "  -p, --preserve       try to preserve file attributes\n"
 			 "  -P, --parents        append source path to destination\n"
 			 "  -q, --quiet          overrides --verbose\n"
-			 "  -r, --recursive      get directories recursively\n"
+			 "  -r, --recursive      transfer directories recursively\n"
 			 "  -R, --resume         resume broken download (restart at eof)\n"
 			 "  -s, --skip-existing  skip file if destination exists\n"
 			 "  -t, --tagged         transfer tagged files\n"
+			 "  -T, --target=HOST    specify target host\n"
 			 "      --type=TYPE      specify transfer type, 'ascii' or 'binary'\n"
 			 "  -u, --unique         store in unique filename (if target supports STOU)\n"
 			 "  -v, --verbose        explain what is being done\n"
@@ -131,7 +124,7 @@ static int do_the_fxp(Ftp *srcftp, const char *src,
 	else if(test(opt, FXP_BINARY))
 		type = tmBinary;
 
-#if defined(HAVE_SETPROCTITLE) || defined(linux)
+#if 0 && (defined(HAVE_SETPROCTITLE) || defined(linux))
 	if(gvUseEnvString && ftp_connected())
 		setproctitle("%s, fxp %s", srcftp->url->hostname, src);
 #endif
@@ -139,7 +132,7 @@ static int do_the_fxp(Ftp *srcftp, const char *src,
 		printf("%s\n", src);
 	}
 	r = ftp_fxpfile(srcftp, src, destftp, dest, how, type);
-#if defined(HAVE_SETPROCTITLE) || defined(linux)
+#if 0 && (defined(HAVE_SETPROCTITLE) || defined(linux))
 	if(gvUseEnvString && ftp_connected())
 		setproctitle("%s", srcftp->url->hostname);
 #endif
@@ -183,7 +176,9 @@ static int fxpfile(const rfile *fi, unsigned int opt,
 		char *p = base_dir_xptr(fi->path);
 		asprintf(&dest, "%s/%s/%s", output, p, base_name_ptr(fi->path));
 		xfree(p);
-	} else
+	} else if(test(opt, FXP_OUTPUT_FILE))
+		dest = xstrdup(output);
+	else
 		asprintf(&dest, "%s/%s", output, base_name_ptr(fi->path));
 
 	path_collapse(dest);
@@ -713,6 +708,12 @@ void cmd_fxp(int argc, char **argv)
 	if(test(opt, FXP_FORCE))
 		opt &= ~FXP_INTERACTIVE;
 
+	if(fxp_output && !test(opt, FXP_RECURSIVE) && list_numitem(gl) +
+	   (test(opt, FXP_TAGGED) ? list_numitem(ftp->taglist) : 0) == 1)
+		{
+			opt |= FXP_OUTPUT_FILE;
+		}
+
 	gvInTransfer = true;
 	gvInterrupted = false;
 
