@@ -29,20 +29,12 @@
 #include "transfer.h"
 #include "commands.h"
 #include <modechange.h>  /* in ../lib/ */
+#include "utils.h"
 
 #ifdef HAVE_REGEX_H
 # include <regex.h>
 #endif
 
-/* in commands.c */
-listitem *ftplist_search(const char *str);
-
-/* in cmd.c */
-void reset_xterm_title(void);
-
-char *human_size(long size);
-char *human_time(unsigned int secs);
-
 static bool get_quit = false;
 static bool get_owbatch = false;
 static bool get_delbatch = false;
@@ -78,7 +70,7 @@ static void print_get_syntax(void)
 			 "  -m, --mask=GLOB      get only files matching GLOB pattern\n"
 			 "  -M, --rx-mask=REGEXP get only files matching REGEXP pattern\n"
 			 "  -n, --newer          get file if remote is newer than local file\n"
-			 "  -o, --output=DEST    store in local directory DEST\n"
+			 "  -o, --output=DEST    store in local file/directory DEST\n"
 			 "  -p, --preserve       try to preserve file attributes and timestamps\n"
 			 "  -P, --parents        append source path to destination\n"
 			 "  -q, --quiet          overrides --verbose\n"
@@ -113,7 +105,7 @@ static int do_the_get(const char *src, const char *dest,
 	getcwd(tmp, PATH_MAX);
 	fulldest = path_absolute(dest, tmp, 0);
 
-#if defined(HAVE_SETPROCTITLE) || defined(linux)
+#if 0 && (defined(HAVE_SETPROCTITLE) || defined(linux))
 	if(gvUseEnvString && ftp_connected())
 		setproctitle("%s, get %s", ftp->url->hostname, src);
 #endif
@@ -136,30 +128,13 @@ static int do_the_get(const char *src, const char *dest,
 							  src, ftp_getreply(false));
 	}
 	xfree(fulldest);
-#if defined(HAVE_SETPROCTITLE) || defined(linux)
+#if 0 && (defined(HAVE_SETPROCTITLE) || defined(linux))
 	if(gvUseEnvString && ftp_connected())
 		setproctitle("%s", ftp->url->hostname);
 #endif
 	return r;
 }
 
-char *make_unique_filename(const char *path)
-{
-	char *f = (char *)xmalloc(strlen(path)+10);
-	char *ext;
-	unsigned n = 0;
-
-	strcpy(f, path);
-	ext = f + strlen(f);
-
-	while(1) {
-		if(access(f, F_OK) != 0)
-			break;
-		sprintf(ext, ".%u", ++n);
-	}
-	return f;
-}
-
 static void get_preserve_attribs(const rfile *fi, char *dest)
 {
 	time_t t;
@@ -210,18 +185,37 @@ static int getfile(const rfile *fi, unsigned int opt,
 		char *apath = base_dir_xptr(fi->path);
 		asprintf(&dest, "%s%s/%s", output, apath, destname);
 		xfree(apath);
-	} else
+	} else if(test(opt, GET_OUTPUT_FILE))
+		dest = xstrdup(output);
+	else
 		asprintf(&dest, "%s/%s", output, destname);
 
 	/* make sure destination directory exists */
 	{
 		char *destdir = base_dir_xptr(dest);
 		int r;
-		r = make_path(destdir, S_IRWXU, S_IRWXU, -1, -1, 1, 0);
-		xfree(destdir);
-		if(r != 0) {
-			transfer_mail_msg(_("failed to create directory %s\n"), destdir);
-			return -1;
+
+		if(destdir) {
+			r = make_path(destdir, S_IRWXU, S_IRWXU, -1, -1, 1, 0);
+			if(r != 0) {
+				transfer_mail_msg(_("failed to create directory %s\n"), destdir);
+				xfree(destdir);
+				return -1;
+			}
+			/* change permission and group, if requested */
+			if(test(opt, GET_CHMOD)) {
+				if(stat(destdir, &sb) == 0) {
+					mode_t m = sb.st_mode;
+					m = mode_adjust(m, cmod);
+					if(chmod(destdir, m) != 0)
+						perror(destdir);
+				}
+			}
+			if(test(opt, GET_CHGRP)) {
+				if(chown(destdir, -1, group_change) != 0)
+					perror(dest);
+			}
+			xfree(destdir);
 		}
 	}
 
@@ -771,6 +765,12 @@ void cmd_get(int argc, char **argv)
 	if(test(opt, GET_FORCE))
 		opt &= ~GET_INTERACTIVE;
 
+	if(get_output && !test(opt, GET_RECURSIVE) && list_numitem(gl) +
+	   (test(opt, GET_TAGGED) ? list_numitem(ftp->taglist) : 0) == 1)
+		{
+			opt |= GET_OUTPUT_FILE;
+		}
+
 	gvInTransfer = true;
 	gvInterrupted = false;
 
