@@ -28,14 +28,12 @@
 #include "input.h"
 #include "commands.h"
 #include "lglob.h"
+#include "utils.h"
 
 #ifdef HAVE_REGEX_H
 # include <regex.h>
 #endif
 
-/* in cmd.c */
-void reset_xterm_title(void);
-
 static bool put_batch = false;
 static bool put_owbatch = false;
 static bool put_delbatch = false;
@@ -71,7 +69,7 @@ static void print_put_syntax(void)
 			 "  -p, --preserve       try to preserve file attributes\n"
 			 "  -P, --parents        append source path to destination\n"
 			 "  -q, --quiet          overrides --verbose\n"
-			 "  -r, --recursive      get directories recursively\n"
+			 "  -r, --recursive      upload directories recursively\n"
 			 "  -R, --resume         resume broken transfer (restart at EOF)\n"
 			 "  -s, --skip-existing  always skip existing files\n"
 			 "  -t, --tagged         transfer (locally) tagged file(s)\n"
@@ -81,36 +79,6 @@ static void print_put_syntax(void)
 			 "      --help           display this help\n"));
 }
 
-char* get_mode_string(mode_t m)
-{
-	static char tmp[4];
-
-	strcpy(tmp, "000");
-
-	if(test(m, S_IRUSR))
-		tmp[0] += 4;
-	if(test(m, S_IWUSR))
-		tmp[0] += 2;
-	if(test(m, S_IXUSR))
-		tmp[0]++;
-
-	if(test(m, S_IRGRP))
-		tmp[1] += 4;
-	if(test(m, S_IWGRP))
-		tmp[1] += 2;
-	if(test(m, S_IXGRP))
-		tmp[1]++;
-
-	if(test(m, S_IROTH))
-		tmp[2] += 4;
-	if(test(m, S_IWOTH))
-		tmp[2] += 2;
-	if(test(m, S_IXOTH))
-		tmp[2]++;
-
-	return tmp;
-}
-
 static int do_the_put(const char *src, const char *dest,
 					  putmode_t how, unsigned opt)
 {
@@ -126,13 +94,13 @@ static int do_the_put(const char *src, const char *dest,
 	else if(test(opt, PUT_BINARY))
 		type = tmBinary;
 
-#if defined(HAVE_SETPROCTITLE) || defined(linux)
+#if 0 && (defined(HAVE_SETPROCTITLE) || defined(linux))
 	if(gvUseEnvString && ftp_connected())
 		setproctitle("%s, put %s", ftp->url->hostname, src);
 #endif
 	r = ftp_putfile(src, dest, how, type,
 					test(opt, PUT_VERBOSE) ? transfer : 0);
-#if defined(HAVE_SETPROCTITLE) || defined(linux)
+#if 0 && (defined(HAVE_SETPROCTITLE) || defined(linux))
 	if(gvUseEnvString && ftp_connected())
 		setproctitle("%s", ftp->url->hostname);
 #endif
@@ -171,7 +139,9 @@ static void putfile(const char *path, struct stat *sb,
 		char *p = base_dir_xptr(path);
 		asprintf(&dest, "%s/%s/%s", output, p, base_name_ptr(path));
 		xfree(p);
-	} else
+	} else if(test(opt, PUT_OUTPUT_FILE))
+		dest = xstrdup(output);
+	else
 		asprintf(&dest, "%s/%s", output, base_name_ptr(path));
 
 	path_collapse(dest);
@@ -290,13 +260,6 @@ static void putfile(const char *path, struct stat *sb,
 	}
 }
 
-char *get_local_curdir(void)
-{
-	static char buf[PATH_MAX+1];
-	getcwd(buf, PATH_MAX);
-	return buf;
-}
-
 static void putfiles(const list *gl, unsigned opt, const char *output)
 {
 	struct stat sb;
@@ -634,6 +597,12 @@ void cmd_put(int argc, char **argv)
 	if(test(opt, PUT_FORCE))
 		opt &= ~PUT_INTERACTIVE;
 
+	if(put_output && !test(opt, PUT_RECURSIVE) && list_numitem(gl) +
+	   (test(opt, PUT_TAGGED) ? list_numitem(gvLocalTagList) : 0) == 1)
+		{
+			opt |= PUT_OUTPUT_FILE;
+		}
+	
 	gvInTransfer = true;
 	gvInterrupted = false;
 
@@ -679,7 +648,7 @@ void cmd_put(int argc, char **argv)
 		reset_xterm_title();
 		exit(0);
 	}
-
+	
 	putfiles(gl, opt, put_output);
 	list_free(gl);
 	if(test(opt, PUT_TAGGED)) {
