@@ -26,6 +26,7 @@
 #include "alias.h"
 #include "commands.h"
 #include "transfer.h"
+#include "utils.h"
 
 static int nerr = 0;
 
@@ -195,11 +196,18 @@ static void parse_host(int trig, FILE *fp)
 		} else if(strcasecmp(e, "port") == 0) {
 			NEXTSTR;
 			url_setport(up, atoi(e));
-		} else if(strcasecmp(e, "nokrb") == 0) {
-			up->nokrb = 1;
+		} else if(strcasecmp(e, "mech") == 0) {
+			NEXTSTR;
+			url_setmech(up, e);
 		} else if(strcasecmp(e, "prot") == 0) {
 			NEXTSTR;
 			url_setprotlevel(up, e);
+		} else if(strcasecmp(e, "passive") == 0) {
+			bool b;
+			NEXTSTR;
+			b = str2bool(e);
+			if(b != -1)
+				url_setpassive(up, b);
 		} else if(strcasecmp(e, "macdef") == 0) {
 			while(e) { /* FIXME: macdef: this is not really true */
 				NEXTSTR;
@@ -213,70 +221,23 @@ static void parse_host(int trig, FILE *fp)
 	}
 
 	if(trig == TRIG_MACHINE) {
-		list *ls;
 		listitem *li;
-		char *tmp;
-
-		asprintf(&tmp, "%s/bookmarks", gvWorkingDirectory);
-		if(strcmp(current_rcfile, tmp) == 0)
-			ls = gvBookmarks;
-		else
-			ls = gvUrlHistory;
-		xfree(tmp);
-
-		li = list_search(ls, (listsearchfunc)urlcmp, up);
-#if 0
-		if(li && !gvReplaceBookmarks) {
-			errp(_("Multiple definitions for '%s', skipping...\n"),
-					 up->alias ? up->alias : up->hostname);
-			url_destroy(up);
-		} else
-#endif
-			{
-			if(li)
-				list_delitem(ls, li);
-			list_additem(ls, up);
-		}
+		li = list_search(gvBookmarks, (listsearchfunc)urlcmp, up);
+		if(li)
+			/* bookmark already exists, overwrite it (delete and create new) */
+			list_delitem(gvBookmarks, li);
+		list_additem(gvBookmarks, up);
 	} else if(trig == TRIG_LOCAL) {
-#if 0
-		if(gvLocalUrl && !gvReplaceBookmarks) {
-			errp(_("Multiple definitions for 'local', skipping...\n"));
-			url_destroy(up);
-		} else
-#endif
-			{
-			if(gvLocalUrl)
-				url_destroy(gvLocalUrl);
-			gvLocalUrl = up;
-		}
+		if(gvLocalUrl)
+			url_destroy(gvLocalUrl);
+		gvLocalUrl = up;
 	} else { /* trig == TRIG_DEFAULT */
-#if 0
-		if(gvDefaultUrl && !gvReplaceBookmarks) {
-			errp(_("Multiple definitions for 'default', skipping...\n"));
-			url_destroy(up);
-		} else
-#endif
-			{
-			if(gvDefaultUrl)
-				url_destroy(gvDefaultUrl);
-			gvDefaultUrl = up;
-		}
+		if(gvDefaultUrl)
+			url_destroy(gvDefaultUrl);
+		gvDefaultUrl = up;
 	}
 }
 
-static void add_to_proxy_exclude(const char *str)
-{
-	char *e;
-	char *s, *orgs;
-
-	orgs = s = xstrdup(str);
-	while((e = strqsep(&s, ':')) != 0) {
-		if(list_search(gvProxyExclude, (listsearchfunc)strcmp, e) == 0)
-			list_additem(gvProxyExclude, xstrdup(e));
-	}
-	xfree(orgs);
-}
-
 int parse_rc(const char *file, bool warn)
 {
 	FILE *fp;
@@ -290,10 +251,9 @@ int parse_rc(const char *file, bool warn)
 		xfree(e);
 		return -1;
 	}
-	xfree(e);
+	current_rcfile = e;
 
 	nerr = 0;
-	current_rcfile = xstrdup(file);
 
 	while(!feof(fp)) {
 		if(nerr>20) {
@@ -370,6 +330,11 @@ int parse_rc(const char *file, bool warn)
 				gvDefaultType = tmAscii;
 			else
 				errp(_("Unknown default_type parameter '%s'... (use 'ascii' or 'binary')\n"), e);
+		} else if(strcasecmp(e, "default_mechanism") == 0) {
+			NEXTSTR;
+			list_free(gvDefaultMechanism);
+			gvDefaultMechanism = list_new((listfunc)xfree);
+			listify_string(e, gvDefaultMechanism);
 		} else if(strcasecmp(e, "anon_password") == 0) {
 			NEXTSTR;
 			xfree(gvAnonPasswd);
@@ -400,10 +365,17 @@ int parse_rc(const char *file, bool warn)
 			NEXTSTR;
 			gvConnectionTimeout = (unsigned)atoi(e);
 		} else if(strcasecmp(e, "include") == 0) {
+			char *rcfile;
 			NEXTSTR;
-			xfree(current_rcfile);
-			parse_rc(e, true);
-			current_rcfile = xstrdup(file);
+			rcfile = tilde_expand_home(e, gvLocalHomeDir);
+			if(strcmp(rcfile, current_rcfile) == 0) {
+				xfree(rcfile);
+				errp(_("Skipping circular include statement: %s\n"), e);
+			} else {
+				xfree(current_rcfile);
+				parse_rc(e, true);
+				current_rcfile = rcfile;
+			}
 		} else if(strcasecmp(e, "prompt1") == 0) {
 			NEXTSTR;
 			xfree(gvPrompt1);
@@ -442,6 +414,11 @@ int parse_rc(const char *file, bool warn)
 			xfree(gvTransferString);
 			gvTransferString = xstrdup(e);
 			unquote_escapes(gvTransferString);
+		} else if(strcasecmp(e, "transfer_xterm_string") == 0) {
+			NEXTSTR;
+			xfree(gvTransferXtermString);
+			gvTransferXtermString = xstrdup(e);
+			unquote_escapes(gvTransferXtermString);
 		} else if(strcasecmp(e, "transfer_end_string") == 0) {
 			NEXTSTR;
 			xfree(gvTransferEndString);
@@ -464,7 +441,7 @@ int parse_rc(const char *file, bool warn)
 			}
 		} else if(strcasecmp(e, "ascii_transfer_mask") == 0) {
 			NEXTSTR;
-			add_ascii_transfer_masks(e);
+			listify_string(e, gvAsciiMasks);
 		} else if(strcasecmp(e, "startup_local_directory") == 0) {
 			NEXTSTR;
 			e = tilde_expand_home(e, gvLocalHomeDir);
@@ -501,7 +478,7 @@ int parse_rc(const char *file, bool warn)
 			gvProxyUrl = url_init(e);
 		} else if(strcasecmp(e, "proxy_exclude") == 0) {
 			NEXTSTR;
-			add_to_proxy_exclude(e);
+			listify_string(e, gvProxyExclude);
 		}
 		else if(strcasecmp(e, "machine") == 0)
 			parse_host(TRIG_MACHINE, fp);
@@ -522,7 +499,7 @@ static url_t *get_autologin_url_short(const char *host)
 	url_t *x, *found = 0;
 	listitem *li;
 
-	li = gvHostCompletion->first;
+	li = gvBookmarks->first;
 	while(li) {
 		x = (url_t *)li->data;
 		li = li->next;
@@ -543,7 +520,7 @@ static url_t *get_autologin_url_short(const char *host)
 		return found;
 
 	/* now do the same for hostnames (skip aliases) */
-	li = gvHostCompletion->first;
+	li = gvBookmarks->first;
 	while(li) {
 		x = (url_t *)li->data;
 		li = li->next;
@@ -588,7 +565,7 @@ url_t *get_autologin_url(const char *host)
 
 	/* try to match as much as possible of the domain name */
 	while((dot = strchr(dot, '.')) != 0) {
-		li = gvHostCompletion->first;
+		li = gvBookmarks->first;
 		while(li) {
 			x = (url_t *)li->data;
 
